/* $Workfile: riport.cpp $ */
   
/*
 * This file demonstrates IO techniques for interfacing
 * to the RIEGL parallel port.
 *
 *    Company: RIEGL Laser Measurement Systems
 * Originator: Roland Schwarz
 *    $Author: rs $
 *      Start: 06.07.98
 *      $Date: 2003/06/11 06:19:36 $
 *   $Modtime: 15.07.98 9:30 $
 */

#include "riport.h"

#include <stdio.h>
#include <windows.h>

#if !(defined(RIPORT_MODE) || defined(BYTE_MODE) || defined(ECP_MODE))
// default if nothing defined on the command line:
#define RIPORT_MODE /* uncomment this if you want to use the riport driver */
//#define BYTE_MODE     /* uncomment this if you want to use direct IO in byte mode */
//#define ECP_MODE    /* uncomment this if you want to use direct IO to the ECP port */
#endif
//
//
// Bit definitions for the DSR.
//

#define DSR_NOT_BUSY            0x80
#define DSR_NOT_ACK             0x40
#define DSR_PERROR              0x20
#define DSR_SELECT              0x10
#define DSR_NOT_FAULT           0x08

//
// More bit definitions for the DSR.
//

#define DSR_NOT_PTR_BUSY        0x80
#define DSR_NOT_PERIPH_ACK      0x80
#define DSR_WAIT                0x80
#define DSR_PTR_CLK             0x40
#define DSR_PERIPH_CLK          0x40
#define DSR_INTR                0x40
#define DSR_ACK_DATA_REQ        0x20
#define DSR_NOT_ACK_REVERSE     0x20
#define DSR_XFLAG               0x10
#define DSR_NOT_DATA_AVAIL      0x08
#define DSR_NOT_PERIPH_REQUEST  0x08

//
// Bit definitions for the DCR.
//

#define DCR_RESERVED            0xC0
#define DCR_DIRECTION           0x20
#define DCR_ACKINT_ENABLED      0x10
#define DCR_SELECT_IN           0x08
#define DCR_NOT_INIT            0x04
#define DCR_AUTOFEED            0x02
#define DCR_STROBE              0x01

//
// More bit definitions for the DCR.
//

#define DCR_NOT_1284_ACTIVE     0x08
#define DCR_ASTRB               0x08
#define DCR_NOT_REVERSE_REQUEST 0x04
#define DCR_NOT_HOST_BUSY       0x02
#define DCR_NOT_HOST_ACK        0x02
#define DCR_DSTRB               0x02
#define DCR_NOT_HOST_CLK        0x01
#define DCR_WRITE               0x01

//
// Bit definitions for ECR register.
//

#define ECR_MODE_MASK           0xE0
#define ECR_SPP_MODE            0x00
#define ECR_BYTE_MODE           0x20
#define ECR_FASTCENT_MODE       0x40
#define ECR_ECP_MODE            0x60
#define ECR_EPP_MODE            0x80
#define ECR_TEST_MODE           0xC0

#define ECR_CONFIG_MODE         0xE0
#define ECR_ERRINT_DISABLED     0x10
#define ECR_DMA_ENABLED         0x08
#define ECR_SERVICE_INTERRUPT   0x04

#define ECR_FIFO_FULL           0x02
#define ECR_FIFO_EMPTY          0x01

//
// Full Mode definitions
//
#define ECP_PS2_MODE            0x34
#define ECP_COMPAT_MODE         0x14


#ifdef RIPORT_MODE
/***************************************************************************************/

char* pszMode = "device driver mode";

static HANDLE	hDevice = INVALID_HANDLE_VALUE;		// Handle to device opened in driver.
static char *sLinkName = "\\\\.\\RiPT";

BOOL RiPortOpen(int nUnit)
{
	char buf[512];
	sprintf(buf, "%s%d", sLinkName, nUnit);

	if (hDevice == INVALID_HANDLE_VALUE)
	{// Create a handle to the driver
		hDevice = CreateFile(buf,
							 GENERIC_READ | GENERIC_WRITE,
							 FILE_SHARE_READ,
							 NULL,
							 OPEN_EXISTING,
							 0,
							 NULL);

		if (hDevice == INVALID_HANDLE_VALUE) return FALSE;
	}
	
	return TRUE;
}

BOOL RiPortClose(void)
{
BOOL bSuccess = TRUE;
	if (hDevice != INVALID_HANDLE_VALUE)
	{// Close the handle to the driver
		bSuccess = CloseHandle(hDevice);
		hDevice = INVALID_HANDLE_VALUE;
	}
	return bSuccess;
}

BOOL RiPortRead(
	LPVOID lpBuffer,
	DWORD nNumberOfBytesToRead,
	LPDWORD lpNumberOfBytesRead)
{// Read requested number of bytes from driver
	return ReadFile(
		hDevice, 
		lpBuffer, 
		nNumberOfBytesToRead, 
		lpNumberOfBytesRead, 
		NULL);
}

#endif // RIPORT_MODE

#ifdef BYTE_MODE
/***************************************************************************************/
#include <conio.h>

char* pszMode = "direct IO - polled bidirectional byte mode";

static unsigned port_base = 0; // no port selected
#define port_data	(port_base+0)
#define port_dsr	(port_base+1)
#define port_dcr	(port_base+2)
#define port_ecr	(port_base+0x402) // register only needed for switching to bidir mode

BOOL RiPortOpen(int nUnit)
{
	// if you are running on WinNT and have the giveio driver
	// installed, you can use this protocol mode too.
	HANDLE hDevice = CreateFile("\\\\.\\giveio",
							 GENERIC_READ | GENERIC_WRITE,
							 FILE_SHARE_READ,
							 NULL,
							 OPEN_EXISTING,
							 0,
							 NULL);
	if (hDevice != INVALID_HANDLE_VALUE)
	{// Close the handle to the driver
		CloseHandle(hDevice);
	}

	if (0==port_base)
	{
		if      (0==nUnit) port_base = 0x378; // LPT1
		else if (1==nUnit) port_base = 0x278; // LPT2
		else               port_base = 0;     // no port selected
	}
	else
		return TRUE; // already opened
	
	if (port_base != 0)
	{
		_outp(port_ecr, ECP_PS2_MODE);				 // PS2-style byte mode, no IRQ's
		_outp(port_dcr, DCR_DIRECTION|DCR_AUTOFEED); // switch to reverse mode
		return TRUE;
	}
	else
		return FALSE;
}

BOOL RiPortClose(void)
{
BOOL bSuccess = TRUE;
	if (port_base != 0)
	{
		_outp(port_dcr, DCR_NOT_INIT);	  // set status lines
		_outp(port_ecr, ECP_COMPAT_MODE); // enter compatibility mode
		bSuccess = TRUE;
		port_base = 0;
	}
	return bSuccess;
}

BOOL RiPortRead(
	LPVOID lpBuffer,
	DWORD nNumberOfBytesToRead,
	LPDWORD lpNumberOfBytesRead)
{// Read requested number of bytes
	*lpNumberOfBytesRead = 0;
	if (port_base!=0)
	{
		while(*lpNumberOfBytesRead < nNumberOfBytesToRead)
		{
			do {/*TODO: add timeout handling*/} 
				while(0!=(_inp(port_dsr) & DSR_NOT_ACK));
			_outp(port_dcr, DCR_DIRECTION);
			do {/*TODO: add timeout handling*/} 
				while(0==(_inp(port_dsr) & DSR_NOT_ACK));
			*( (BYTE*)lpBuffer + *lpNumberOfBytesRead ) = (BYTE)_inp(port_data);
			(*lpNumberOfBytesRead)++;
			_outp(port_dcr, DCR_DIRECTION|DCR_AUTOFEED);
		}
		return TRUE;
	}
	else
		return FALSE;
}

#endif // BYTE_MODE

#ifdef ECP_MODE
/***************************************************************************************/
#include <conio.h>

char* pszMode = "direct IO - polled ECP mode";

static unsigned port_base = 0; // no port selected
#define port_data	(port_base+0)
#define port_dsr	(port_base+1)
#define port_dcr	(port_base+2)
#define port_ecr	(port_base+0x402)
#define port_fifo	(port_base+0x400)
 
BOOL RiPortOpen(int nUnit)
{
	// if you are running on WinNT and have the giveio driver
	// installed, you can use this protocol mode too.
	HANDLE hDevice = CreateFile("\\\\.\\giveio",
							 GENERIC_READ | GENERIC_WRITE,
							 FILE_SHARE_READ,
							 NULL,
							 OPEN_EXISTING,
							 0,
							 NULL);
	if (hDevice != INVALID_HANDLE_VALUE)
	{// Close the handle to the driver
		CloseHandle(hDevice);
	}

	if (0==port_base)
	{
		if      (0==nUnit) port_base = 0x378; // LPT1
		else if (1==nUnit) port_base = 0x278; // LPT2
		else               port_base = 0;     // no port selected
	}
	else
		return TRUE; // already opened
	
	if (port_base != 0)
	{
		// switch to compatibility mode
		_outp(port_ecr, 
			ECR_SPP_MODE|ECR_ERRINT_DISABLED|ECR_SERVICE_INTERRUPT);
		_outp(port_dcr,
			DCR_NOT_REVERSE_REQUEST|DCR_NOT_1284_ACTIVE);
		// switch to reverse direction
		_outp(port_ecr,
			ECR_BYTE_MODE|ECR_ERRINT_DISABLED|ECR_SERVICE_INTERRUPT);
		_outp(port_dcr,
			DCR_DIRECTION|DCR_NOT_REVERSE_REQUEST);
		_outp(port_ecr,
			ECR_ECP_MODE|ECR_ERRINT_DISABLED|ECR_SERVICE_INTERRUPT);
		_outp(port_dcr,
			DCR_DIRECTION);
		return TRUE;
	}
	else
		return FALSE;
}

BOOL RiPortClose(void)
{
BOOL bSuccess = TRUE;
	if (port_base != 0)
	{
		// switch to compatibility mode
		_outp(port_ecr,
			ECR_SPP_MODE|ECR_ERRINT_DISABLED|ECR_SERVICE_INTERRUPT);
		_outp(port_dcr,
			DCR_NOT_REVERSE_REQUEST|DCR_NOT_1284_ACTIVE);
		
		bSuccess = TRUE;
		port_base = 0;
	}
	return bSuccess;
}

BOOL RiPortRead(
	LPVOID lpBuffer,
	DWORD nNumberOfBytesToRead,
	LPDWORD lpNumberOfBytesRead)
{// Read requested number of bytes
	*lpNumberOfBytesRead = 0;
	if (port_base!=0)
	{
		while(*lpNumberOfBytesRead < nNumberOfBytesToRead)
		{
			do {/*TODO: add timeout handling*/} 
				while(_inp(port_ecr) & ECR_FIFO_EMPTY);
			// read a byte
			*( (BYTE*)lpBuffer + *lpNumberOfBytesRead ) = (BYTE)_inp(port_fifo);
			(*lpNumberOfBytesRead)++;
		}
		return TRUE;
	}
	else
		return FALSE;
}

#endif // ECP_MODE
